package ssw.mj.test;

import java.io.StringReader;

import junit.framework.TestCase;

import ssw.mj.Parser;
import ssw.mj.Scanner;
import ssw.mj.Token;
import ssw.mj.codegen.Code;
import ssw.mj.symtab.Obj;
import ssw.mj.symtab.Struct;
import ssw.mj.symtab.Tab;

/**
 * Test cases for the <code>Parser</code> class.
 * 
 * @author AW
 */
public class ParserTest extends TestCase {
	private static final String CR = "\r";
	private static final String LF = "\n";
	
	private static final int
		none       = Token.none,
		ident      = Token.ident,
		number     = Token.number,
		charConst  = Token.charConst,
		plus       = Token.plus,
		minus      = Token.minus,
		times      = Token.times,
		slash      = Token.slash,
		rem        = Token.rem,
		eql        = Token.eql,
		neq        = Token.neq,
		gtr        = Token.gtr,
		geq        = Token.geq,
		less       = Token.less,
		leq        = Token.leq,
		and        = Token.and,
		or         = Token.or,
		assign     = Token.assign,
		pplus      = Token.pplus,
		mminus     = Token.mminus,
		semicolon  = Token.semicolon,
		comma      = Token.comma,
		period     = Token.period,
		lpar       = Token.lpar,
		rpar       = Token.rpar,
		lbrack     = Token.lbrack,
		rbrack     = Token.rbrack,
		lbrace     = Token.lbrace,
		rbrace     = Token.rbrace,
		classKW    = Token.classKW,
		continueKW = Token.continueKW,
		doKW       = Token.doKW,
		elseKW     = Token.elseKW,
		finalKW    = Token.finalKW,
		ifKW       = Token.ifKW,
		newKW      = Token.newKW,
		printKW    = Token.printKW,
		readKW     = Token.readKW,
		returnKW   = Token.returnKW,
		voidKW     = Token.voidKW,
		whileKW    = Token.whileKW,
		eof        = Token.eof;

	private static TestPrintWriter output;
	private static TestByteArrayOutputStream code;

	// ---------- helpers ----------
	
	private void initParser(String program) {
		output = new TestPrintWriter();
		Scanner.init(new StringReader(program), output);
		code = new TestByteArrayOutputStream();
	}

	// ----------------------------------------------------
	// ---------- test complete correct programs ----------

	/** Test the shortest valid program: */
	public void testShortestProgram () {
		initParser("class Shortest { }");

		Parser.parse(output);
		output.addExpectedLine("-- line 1, col 18 (SEM): 'main' not found");
		output.verify();

		////////////////////////		
		// check symbol table //
		////////////////////////
		Obj actual, expected;

		// just to make sure that the Universe is also there		
		actual = Tab.find("eol");
		expected = new Obj(Obj.Con, "eol", Tab.charType, 10, 0, null);
		assertEquals("Universe 'eol'", expected, actual);

		// only name in symbol table of this program
		actual = Tab.find("Shortest");
		expected = new Obj(Obj.Prog, "Shortest", Tab.noType, 0, 0, null);
		assertEquals("Program 'Shortest'", expected, actual);

		// make sure that this is the end
		assertNull(actual.next);
		
		//////////////////////////		
		// check generated code //
		//////////////////////////
		code.setCodeSize(0);
		Code.write(code);
		code.verify();
	}

	public void testScriptExample () {
		initParser("class P" + LF +
		           "" + LF +
		           "\tfinal int size = 10;" + LF +
		           "" + LF +
		/*line  5*/"\tclass Table {" + LF +
		           "\t\tint[] pos;" + LF +
		           "\t\tint[] neg;" + LF +
		           "\t}" + LF +
		           "" + LF +
		/*line 10*/"\tTable val;" + LF +
		           "{" + LF +
		           "\tvoid main()" + LF +
		           "\t\tint x, i;" + LF +
		           "\t{ /*---------- Initialize val */" + LF +
		/*line 15*/"\t\tval = new Table;" + LF +
		           "\t\tval.pos = new int[size]; val.neg = new int[size];" + LF +
		           "\t\ti = 0;" + LF +
		           "\t\twhile (i < size) {" + LF +
		           "\t\t\tval.pos[i] = 0; val.neg[i] = 0;" + LF +
		/*line 20*/"\t\t\ti++;" + LF +
		           "\t\t}" + LF +
		           "\t\t/*---------- Read values */" + LF +
		           "\t\tread(x);" + LF +
		           "\t\twhile (x > 0) {" + LF +
		/*line 25*/"\t\t\tif (0 <= x && x < size) {" + LF +
		           "\t\t\t\tval.pos[x]++;" + LF +
		           "\t\t\t} else if (-size < x && x < 0) {" + LF +
		           "\t\t\t\tval.neg[-x]++;" + LF +
		           "\t\t\t}" + LF +
		/*line 30*/"\t\t\tread(x);" + LF +
		           "\t\t}" + LF +
		           "\t}" + LF +
		           "}");
		
		Parser.parse(output);
		output.verify();
		
		////////////////////////
		// check symbol table //
		////////////////////////
		Obj actual, expected;

		actual = Tab.find("P");
		expected = new Obj(Obj.Prog, "P", Tab.noType, 0, 0, null);
		// We cheat here, because we will compare all the rest later.
		expected.locals = actual.locals;
		assertEquals("Program 'P'", expected, actual);
		
		// Knowing the implementation of the symbol table
		// we can traverse it and check all the names separately.
		// NOTE: This gives us more precise error messages
		//       than just re-creating the whole symbol table and
		//       have it check by one call to equals(Prog).
		
		actual = actual.locals;
		expected = new Obj(Obj.Con, "size", Tab.intType, 10, 0, null);
		assertEquals("Constant 'size'", expected, actual);
		
		actual = actual.next;
		Struct type = new Struct(Struct.Class); 
		type.n = 2; 
		// No cheating here, field list is small enough.
		// So we explicitly re-create it and check it at once (with equals).
		type.fields = new Obj(Obj.Fld, "pos", 
							new Struct(Struct.Arr, Tab.intType), 0, 1, null);
		assertEquals("Field 'pos'", type.fields, actual.type.fields);
		type.fields.next = new Obj(Obj.Fld, "neg", 
							new Struct(Struct.Arr, Tab.intType), 1, 1, null);
		assertEquals("Field 'neg'", type.fields.next, actual.type.fields.next);
		expected = new Obj(Obj.Type, "Table", type, 0, 0, null);
		assertEquals("Type 'Table'", expected, actual);
		
		actual = actual.next;
		expected = new Obj(Obj.Var, "val", type, 0, 0, null);
		assertEquals("Global 'val'", expected, actual);
		
		actual = actual.next;
		Obj local = new Obj(Obj.Var, "x", Tab.intType, 0, 1, null);
		assertEquals("Local 'x'", local, actual.locals);
		local.next = new Obj(Obj.Var, "i", Tab.intType, 1, 1, null);
		assertEquals("Local 'i'", local.next, actual.locals.next);
		expected = new Obj(Obj.Meth, "main", Tab.noType, 0, 0, local);
		assertEquals("Method 'main'", expected, actual);
		
		// make sure that this is the end
		assertNull(actual.next);
		
		//////////////////////////
		// check generated code //
		//////////////////////////
			// void main () {
				// val = new Table;                 (6 byte)
		code.expect(32, 1);		// new
		code.expect(8, 2);		// s
		code.expect(12, 1);		// putstatic
		code.expect(0, 2);		// s
				// val.pos = new int[size];         (13 byte)
		code.expect(11, 1);		// getstatic
		code.expect(0, 2);		// s
		code.expect(22, 1);		// const_
		code.expect(10, 4);		// w
		code.expect(33, 1);		// newarray
		code.expect(1, 1);		// b
		code.expect(14, 1);		// putfield
		code.expect(0, 2);		// s
				// val.neg = new int[size];         (13 byte)
		code.expect(11, 1);		// getstatic
		code.expect(0, 2);		// s
		code.expect(22, 1);		// const_
		code.expect(10, 4);		// w
		code.expect(33, 1);		// newarray
		code.expect(1, 1);		// b
		code.expect(14, 1);		// putfield
		code.expect(1, 2);		// s
				// i = 0;                           (2 byte)
		code.expect(15, 1);		// const_0
		code.expect(8, 1);		// store_1
				// while (i < size) {
					// val.pos[i] = 0;                (9 byte)
		code.expect(11, 1);		// getstatic
		code.expect(0, 2);		// s
		code.expect(13, 1);		// getfield
		code.expect(0, 2);		// s
		code.expect(3, 1);		// load_1		
		code.expect(15, 1);		// const_0
		code.expect(35, 1);		// astore
					// val.neg[i] = 0;                (9 byte)
		code.expect(11, 1);		// getstatic
		code.expect(0, 2);		// s
		code.expect(13, 1);		// getfield
		code.expect(1, 2);		// s
		code.expect(3, 1);		// load_1		
		code.expect(15, 1);		// const_0
		code.expect(35, 1);		// astore
					// i++;                           (3 byte)
		code.expect(31, 1);		// inc
		code.expect(1, 1);		// b1
		code.expect(1, 1);		// b2
				// }
				// read(x);                         (2 byte)
		code.expect(53, 1);		// read
		code.expect(7, 1);		// store_0
				// while (x > 0) {
					// if (0 <= x && x < size) {
						// val.pos[x]++;                (12 byte)
		code.expect(11, 1);		// getstatic
		code.expect(0, 2);		// s
		code.expect(13, 1);		// getfield
		code.expect(0, 2);		// s
		code.expect(2, 1);		// load_0
		code.expect(41, 1);		// dup2
		code.expect(34, 1); 	// aload
		code.expect(16, 1);		// const_1
		code.expect(23, 1);		// add
		code.expect(35, 1);		// astore
					// } else 
						// if (-size < x && x < 0) {    (6 byte)
		code.expect(22, 1);		// const
		code.expect(10, 4);		// w
		code.expect(28, 1);   // neg
							// val.neg[-x]++;             (13 byte)
		code.expect(11, 1);		// getstatic
		code.expect(0, 2);		// s
		code.expect(13, 1);		// getfield
		code.expect(1, 2);		// s
		code.expect(2, 1);		// load_0
		code.expect(28, 1);		// neg
		code.expect(41, 1);		// dup2
		code.expect(34, 1); 	// aload
		code.expect(16, 1);		// const_1
		code.expect(23, 1);		// add
		code.expect(35, 1);		// astore
					// read(x);                       (2 byte)
		code.expect(53, 1);		// read
		code.expect(7, 1);		// store_0
				// }
			// }
		
		Code.write(code);

		code.setCodeSize(90);
		code.setDataSize(1);
		code.setMainPC(-1);
//		code.dump();	// dump code buffer (for debugging)
		code.verify();
	}
	
	public void testAllProds () {
		initParser("class AllProds" + LF +
		           "  final int nPoints = 2;" + LF +
		           "  final char exclaim = '!';" + LF +
		           "" + LF +
		/*line  5*/"  class Point {" + LF +
		           "    int x, y;" + LF +
		           "  }" + LF +
		           "" + LF +
		           "  char[] msg;" + LF +
		/*line 10*/"  Point[] pArr;" + LF +
		           "" + LF +
		           "{" + LF +
		           "  void printError ()" + LF +
		           "    int i;" + LF +
		/*line 15*/"  {" + LF +
		           "    /* print error message */" + LF +
		           "    i = 0;" + LF +
		           "    while(i<len(msg)){print(msg[i]);i++;}" + LF +
		           "    print(exclaim);" + LF +
		/*line 20*/"    do {" + LF +
		           "      i--;" + LF +
		           "      if (i > 5) continue;" + LF +
		           "      print(msg[i], 2);" + LF +
		           "    } while(i >= 0);" + LF +
		/*line 25*/"  }" + LF +
		           "" + LF +
		           "  int getX (Point p) { return p.x; }" + LF +
		           "" + LF +
		           "  void setX (int x, Point p) {" + LF +
		/*line 30*/"    if (x > -10 && x <= 9 ||" + LF +
		           "        x == 4*3/4%4+12-11*(2147483647/(-2147483647-1)%2) )" + LF +
		           "      p.x = x;" + LF +
		           "    else printError();" + LF +
		           "  }" + LF +
		/*line 35*/"" + LF +
		           "" + LF +
		           "  void setMsg () {" + LF +
		           "    msg = new char[5];" + LF +
		           "    msg[0] = 'E';" + LF +
		/*line 40*/"    msg[1] = 'r';" + LF +
		           "    msg[2] = 'r';" + LF +
		           "    msg[3] = 'o';" + LF +
		           "    msg[4] = 'r';" + LF +
		           "  }" + LF +
		/*line 45*/"" + LF +
		           "  void setPoints ()" + LF +
		           "    int i;" + LF +
		           "  {" + LF +
		           "    pArr = new Point[nPoints];" + LF +
		/*line 50*/"    i = 0;" + LF +
		           "    while (i < len(pArr)) {" + LF +
		           "      pArr[i] = new Point;" + LF +
		           "      pArr[i].x = i;" + LF +
		           "      pArr[i].y = i;" + LF +
		/*line 55*/"      i++;" + LF +
		           "    }" + LF +
		           "  }" + LF +
		           "" + LF +
		           "  void main ()" + LF +
		/*line 60*/"    int i;" + LF +
		           "  {" + LF +
		           "    setMsg();" + LF +
		           "    setPoints();" + LF +
		           "    read(i);" + LF +
		/*line 65*/"    if (i < nPoints) {" + LF +
		           "      print(getX(pArr[i]));" + LF +
		           "      print(pArr[i].y);" + LF +
		           "    }" + LF +
		           "    else" + LF +
		/*line 70*/"      printError();" + LF +
		           "    print(eol);" + LF +
		           "  }" + LF +
		           "}");

		Parser.parse(output);
		output.verify();

		////////////////////////		
		// check symbol table //
		////////////////////////

		Obj actual, expected;

		actual = Tab.find("AllProds");
		expected = new Obj(Obj.Prog, "AllProds", Tab.noType, 0, 0, null);
		// We cheat here, because we will compare all the rest later.
		expected.locals = actual.locals;
		assertEquals("Program 'AllProds'", expected, actual);
		
		actual = actual.locals;
		expected = new Obj(Obj.Con, "nPoints", Tab.intType, 2, 0, null);
		assertEquals("Constant 'nPoints'", expected, actual);
		
		actual = actual.next;
		expected = new Obj(Obj.Con, "exclaim", Tab.charType, 33, 0, null);
		assertEquals("Constant 'exclaim'", expected, actual);
		
		actual = actual.next;

		Struct pointT = new Struct(Struct.Class); 
		pointT.n = 2; 
		pointT.fields = new Obj(Obj.Fld, "x", Tab.intType, 0, 1, null);
		assertEquals("Field 'x'", pointT.fields, actual.type.fields);
		pointT.fields.next = new Obj(Obj.Fld, "y", Tab.intType, 1, 1, null);
		assertEquals("Field 'y'", pointT.fields.next, actual.type.fields.next);

		expected = new Obj(Obj.Type, "Point", pointT, 0, 0, null);
		assertEquals("Class 'Point'", expected, actual);
		
		actual = actual.next;
		expected = new Obj(Obj.Var, "msg", 
							new Struct(Struct.Arr, Tab.charType), 0, 0, null);
		assertEquals("Global 'msg'", expected, actual);

		actual = actual.next;
		expected = new Obj(Obj.Var, "pArr", 
							new Struct(Struct.Arr, pointT), 1, 0, null);
		assertEquals("Global 'pArr'", expected, actual);

		actual = actual.next;	// printError
		Obj local = new Obj(Obj.Var, "i", Tab.intType, 0, 1, null);
		assertEquals("Local 'i'", local, actual.locals);
		expected = new Obj(Obj.Meth, "printError", Tab.noType, 0, 0, local);
		assertEquals("Method 'printError'", expected, actual);
		
		actual = actual.next;	// getX
		local = new Obj(Obj.Var, "p", pointT, 0, 1, null);
		assertEquals("Local 'p'", local, actual.locals);
		expected = new Obj(Obj.Meth, "getX", Tab.intType, 0, 1, local);
		assertEquals("Method 'getX'", expected, actual);
		
		actual = actual.next; 	// setX
		local = new Obj(Obj.Var, "x", Tab.intType, 0, 1, null);
		assertEquals("Local 'x'", local, actual.locals);
		local.next = new Obj(Obj.Var, "p", pointT, 1, 1, null);
		assertEquals("Local 'p'", local.next, actual.locals.next);
		expected = new Obj(Obj.Meth, "setX", Tab.noType, 0, 2, local);
		assertEquals("Method 'setX'", expected, actual);
		
		actual = actual.next;	// setMsg
		expected = new Obj(Obj.Meth, "setMsg", Tab.noType, 0, 0, null);
		assertEquals("Method 'setMsg'", expected, actual);
		
		actual = actual.next;	// setPoints
		local = new Obj(Obj.Var, "i", Tab.intType, 0, 1, null);
		assertEquals("Local 'i'", local, actual.locals);
		expected = new Obj(Obj.Meth, "setPoints", Tab.noType, 0, 0, local);
		assertEquals("Method 'setPoints'", expected, actual);
		
		actual = actual.next;	// main
		// local is same as above
		assertEquals("Local 'i'", local, actual.locals);
		expected = new Obj(Obj.Meth, "main", Tab.noType, 0, 0, local);
		assertEquals("Method 'main'", expected, actual);
		
		// make sure that this is the end
		assertNull(actual.next);
		
		//////////////////////////
		// check generated code //
		//////////////////////////

			// void printError () {
				// i = 0;                           (2 byte)
		code.expect(15, 1);		// const_0
		code.expect(7, 1);		// store_0
				// while (i < len(msg)) {
					// print(msg[i]);                 (7 byte)
		code.expect(11, 1);		// getstatic
		code.expect(0, 2);		// s
		code.expect(2, 1);		// load_0
		code.expect(36, 1);		// baload
		code.expect(15, 1);		// const_0
		code.expect(56, 1);		// bprint
					// i++;                           (3 byte)
		code.expect(31, 1);		// inc
		code.expect(0, 1);		// b1
		code.expect(1, 1);		// b2
				// }
				// print(exclaim);                  (7 byte)
		code.expect(22, 1);		// const
		code.expect(33, 4);		// w
		code.expect(15, 1);		// const_0	
		code.expect(56, 1);		// bprint
				// do {
					// i--;                           (3 byte)
		code.expect(31, 1);		// inc
		code.expect(0, 1);		// b1
		code.expect(-1, 1);		// b2
					// if (i > 5)
						// continue
					// print(msg[i], 2);              (7 byte)
		code.expect(11, 1);		// getstatic
		code.expect(0, 2);		// s
		code.expect(2, 1);		// load_0
		code.expect(36, 1);		// baload
		code.expect(17, 1);		// const_2
		code.expect(56, 1);		// bprint
				// } while (i >= 0);
			// }
			// int getX (Point p) {
				// return p.x;                      (4 byte)
		code.expect(2, 1);		// load_0
		code.expect(13, 1);		// getfield
		code.expect(0, 2);		// s
			// }
			// void setX (int x, Point p) {
				// if (x > -10 && x <= 9 ||         (6 byte)
		code.expect(22, 1);   // const
		code.expect(10, 4);   // w
		code.expect(28, 1);   // neg
				//     x == 4 * 3 / 4 % 4 + 12 -    (36 byte)
				//          11 * (2147483647 / (-2147483647 - 1) % 2))
		code.expect(19, 1);		// const_4
		code.expect(18, 1);		// const_3
		code.expect(25, 1);		// mul
		code.expect(19, 1);		// const_4
		code.expect(26, 1);		// div
		code.expect(19, 1);		// const_4
		code.expect(27, 1);		// rem
		code.expect(22, 1);		// const
		code.expect(12, 4);		// w
		code.expect(23, 1);		// add
		code.expect(22, 1);		// const
		code.expect(11, 4);		// w
		code.expect(22, 1);		// const
		code.expect(2147483647, 4);	// w
		code.expect(22, 1);		// const
		code.expect(2147483647, 4);	// w
		code.expect(28, 1);		// neg
		code.expect(16, 1);		// const_1
		code.expect(24, 1);		// sub
		code.expect(26, 1);		// div
		code.expect(17, 1);		// const_2
		code.expect(27, 1);		// rem
		code.expect(25, 1);		// mul
		code.expect(24, 1);		// sub
					// p.x = x;                       (5 byte)
		code.expect(3, 1);		// load_1
		code.expect(2, 1);		// load_0
		code.expect(14, 1);		// putfield
		code.expect(0, 2);		// s
				// else
					// printError();
			// }
			// void setMsg () {
				// msg = new char[5];               (6 byte)
		code.expect(20, 1);		// const_5
		code.expect(33, 1);		// newarray
		code.expect(0, 1);		// b
		code.expect(12, 1);		// putstatic
		code.expect(0, 2);		// s
				// msg[0] = 'E';                    (10 byte)
		code.expect(11, 1);		// getstatic
		code.expect(0, 2);		// s
		code.expect(15, 1);		// const_0
		code.expect(22, 1);		// const
		code.expect(69, 4);		// w
		code.expect(37, 1);		// bastore
				// msg[1] = 'r';                    (10 byte)
		code.expect(11, 1);		// getstatic
		code.expect(0, 2);		// s
		code.expect(16, 1);		// const_1
		code.expect(22, 1);		// const
		code.expect(114, 4);	// w
		code.expect(37, 1);		// bastore
				// msg[2] = 'r';                    (10 byte)
		code.expect(11, 1);		// getstatic
		code.expect(0, 2);		// s
		code.expect(17, 1);		// const_2
		code.expect(22, 1);		// const
		code.expect(114, 4);	// w
		code.expect(37, 1);		// bastore
				// msg[3] = 'o';                    (10 byte)
		code.expect(11, 1);		// getstatic
		code.expect(0, 2);		// s
		code.expect(18, 1);		// const_3
		code.expect(22, 1);		// const
		code.expect(111, 4);	// w
		code.expect(37, 1);		// bastore
				// msg[4] = 'r';                    (10 byte)
		code.expect(11, 1);		// getstatic
		code.expect(0, 2);		// s
		code.expect(19, 1);		// const_4
		code.expect(22, 1);		// const
		code.expect(114, 4);	// w
		code.expect(37, 1);		// bastore
			// }
			// void setPoints () {
				// pArr = new Point[nPoints];       (6 byte)
		code.expect(17, 1);		// const_2;
		code.expect(33, 1);		// newarray
		code.expect(1, 1);		// b
		code.expect(12, 1);		// putstatic
		code.expect(1, 2);		// s
				// i = 0;                           (2 byte)
		code.expect(15, 1);		// const_0;
		code.expect(7, 1);		// store_0;
				// while (i < len(pArr)) {
					// pArr[i] = new Point;           (8 byte)
		code.expect(11, 1);		// getstatic
		code.expect(1, 2);		// s
		code.expect(2, 1);		// load_0
		code.expect(32, 1);		// new
		code.expect(8, 2);		// s
		code.expect(35, 1);		// astore
					// pArr[i].x = i;                 (9 byte)
		code.expect(11, 1);		// getstatic
		code.expect(1, 2);		// s
		code.expect(2, 1);		// load_0
		code.expect(34, 1);		// aload
		code.expect(2, 1);		// load_0
		code.expect(14, 1);		// putfield
		code.expect(0, 2);		// s
					// pArr[i].y = i;                 (9 byte)
		code.expect(11, 1);		// getstatic
		code.expect(1, 2);		// s
		code.expect(2, 1);		// load_0
		code.expect(34, 1);		// aload
		code.expect(2, 1);		// load_0
		code.expect(14, 1);		// putfield
		code.expect(1, 2);		// s
					// i++;                           (3 byte)
		code.expect(31, 1);   // inc
		code.expect(0, 1);    // b1
		code.expect(1, 1);    // b2
				// }
			// }
			// void main () {
				// setMsg();
				// setPoints();
				// read(i);                         (2 byte)
		code.expect(53, 1);		// read
		code.expect(7, 1);		// store_0
				// if (i < nPoints) {
					// print(getX(pArr[i]));          (6 byte)
		code.expect(11, 1);   // getstatic
		code.expect(1, 2);    // s
		code.expect(2, 1);    // load_0
		code.expect(15, 1);		// const_0
		code.expect(54, 1);		// print
					// print(pArr[i].y);              (10 byte)
		code.expect(11, 1);		// getstatic
		code.expect(1, 2);		// s
		code.expect(2, 1);		// load_0
		code.expect(34, 1);		// aload
		code.expect(13, 1);		// getfield
		code.expect(1, 2);		// s
		code.expect(15, 1);		// const_0
		code.expect(54, 1);		// print
				// } else
					// printError();
				// print(eol);                      (7 byte)
		code.expect(22, 1);		// const
		code.expect(10, 4);		// w
		code.expect(15, 1);		// const_0
		code.expect(56, 1);		// bprint
			// }
		
		Code.write(code);

		code.setCodeSize(198);
		code.setDataSize(2);
		code.setMainPC(-1);
		code.dump();	// dump code buffer (for debugging)
		code.verify();
	}
	
	// -----------------------------------------------
	// ---------- test programs with errors ----------

	public void testWrongStart () {
		initParser(	"SomeClass { }");
		
		Parser.parse(output);
		output.addExpectedLine("-- line 1, col 1 (SYN): class expected");	
		output.verify();
	}
	
	public void testNoProgName () {
		initParser(	"class { }");
		
		Parser.parse(output);
		output.addExpectedLine("-- line 1, col 7 (SYN): identifier expected");
		output.verify();	
	}

	public void testUndefName () {
		initParser(	"class A" + LF +
					"  integer x;" + LF +
					"{ }");
		
		Parser.parse(output);
		output.addExpectedLine("-- line 2, col 11 (SEM): 'integer' not found");
		output.addExpectedLine("-- line 3, col 3 (SEM): 'main' not found");
		output.verify();	
	}
	
	public static void main (String[] args) {
		junit.textui.TestRunner.run(ParserTest.class);
	}
}
